%global  _hardened_build     1
%global  freenginx_user      freenginx

# Disable strict symbol checks in the link editor.
# See: https://src.fedoraproject.org/rpms/redhat-rpm-config/c/078af19
%undefine _strict_symbol_defs_build

%bcond_with geoip

# freenginx gperftools support should be disabled for RHEL >= 8
# see: https://bugzilla.redhat.com/show_bug.cgi?id=1931402
%if 0%{?rhel} >= 8
%global with_gperftools 0
%else
# gperftools exists only on selected arches
# gperftools *detection* is failing on ppc64*, possibly only configure
# bug, but disable anyway.
%ifnarch s390 s390x ppc64 ppc64le
%global with_gperftools 1
%endif
%endif

%global with_aio 1


%if 0%{?fedora} > 22
%global with_mailcap_mimetypes 1
%endif

# kTLS requires OpenSSL 3.0 (default in F36+ and EL9+, available in EPEL8)
%if 0%{?fedora} >= 36 || 0%{?rhel} >= 8
%global with_ktls 1
%endif

# Build against OpenSSL 1.1 on EL7
%if 0%{?rhel} == 7
%global openssl_pkgversion 11
%endif

# Build against OpenSSL 3 on EL8
%if 0%{?rhel} == 8
%global openssl_pkgversion 3
%endif

%global freenginx_abiversion %{version}

%global freenginx_moduledir %{_libdir}/freenginx/modules
%global freenginx_moduleconfdir %{_datadir}/freenginx/modules
%global freenginx_srcdir %{_usrsrc}/%{name}-%{version}-%{release}

# Do not generate provides/requires from freenginx sources
%global __provides_exclude_from ^%{freenginx_srcdir}/.*$
%global __requires_exclude_from ^%{freenginx_srcdir}/.*$


Name:              freenginx
Epoch:             2
Version:           1.28.0
Release:           1%{?dist}

Summary:           A high performance web server and reverse proxy server
License:           BSD-2-Clause
URL:               https://freenginx.org
Packager:          Teddysun <i@teddysun.com>
Source0:           https://freenginx.org/download/freenginx-%{version}.tar.gz
Source1:           https://freenginx.org/download/freenginx-%{version}.tar.gz.asc
# Keys are found here: https://freenginx.org/en/pgp_keys.html
Source2:           https://freenginx.org/keys/mdounin.key

Source10:          freenginx.service
Source11:          freenginx.logrotate
Source12:          freenginx.conf
Source13:          freenginx-upgrade
Source14:          freenginx-upgrade.8
Source15:          macros.freenginxmods.in
Source16:          freenginxmods.attr
Source17:          freenginx-ssl-pass-dialog
Source18:          freenginx@.service
Source19:          freenginx.sysusers
Source102:         freenginx-logo.png
Source200:         README.dynamic
Source220:         instance.conf

BuildRequires:     make
BuildRequires:     gcc
BuildRequires:     gnupg2
%if 0%{?with_gperftools}
BuildRequires:     gperftools-devel
%endif
BuildRequires:     libxcrypt-devel
BuildRequires:     openssl%{?openssl_pkgversion}-devel
BuildRequires:     pcre2-devel
BuildRequires:     zlib-devel
BuildRequires:     gd-devel
BuildRequires:     libxml2-devel
BuildRequires:     libxslt-devel

Requires:          freenginx-filesystem = %{epoch}:%{version}-%{release}
Requires:          system-logos-httpd

Provides:          webserver
%if 0%{?fedora} || 0%{?rhel} >= 8
Recommends:        logrotate
%endif
Requires:          %{name}-core = %{epoch}:%{version}-%{release}

BuildRequires:     systemd
BuildRequires:     systemd-rpm-macros
%{?systemd_requires}

# For external freenginx modules
Provides:          freenginx(abi) = %{freenginx_abiversion}
# 
Conflicts:         nginx(abi)

%description
freenginx is a web server and a reverse proxy server for HTTP, SMTP, POP3 and
IMAP protocols, with a strong focus on high concurrency, performance and low
memory usage.

%package core
Summary: freenginx minimal core
%if 0%{?with_mailcap_mimetypes}
Requires:          freenginx-mimetypes
%endif
Requires:          openssl%{?openssl_pkgversion}-libs
Requires(pre):     freenginx-filesystem

%description core
freenginx minimal core

%package filesystem
Summary:           The basic directory layout for the freenginx server
BuildArch:         noarch
%{?sysusers_requires_compat}

%description filesystem
The freenginx-filesystem package contains the basic directory layout
for the freenginx server including the correct permissions for the
directories.

%if %{with geoip}
%package mod-http-geoip
Summary:           freenginx HTTP geoip module
BuildRequires:     GeoIP-devel
Requires:          freenginx(abi) = %{freenginx_abiversion}
Requires:          GeoIP

%description mod-http-geoip
%{summary}.
%endif

%package mod-devel
Summary:           freenginx module development files
Requires:          freenginx = %{epoch}:%{version}-%{release}
Requires:          make
Requires:          gcc
Requires:          gd-devel
%if 0%{?with_gperftools}
Requires:          gperftools-devel
%endif
%if %{with geoip}
Requires:          GeoIP-devel
%endif
Requires:          libxslt-devel
Requires:          openssl%{?openssl_pkgversion}-devel
Requires:          pcre2-devel
Requires:          perl-devel
Requires:          perl(ExtUtils::Embed)
Requires:          zlib-devel

%description mod-devel
%{summary}.


%prep
# Combine all keys from upstream into one file
cat %{S:2} > %{_builddir}/%{name}.gpg
%{gpgverify} --keyring='%{_builddir}/%{name}.gpg' --signature='%{SOURCE1}' --data='%{SOURCE0}'
rm -f %{_builddir}/%{name}.gpg
%autosetup -p1
cp %{SOURCE200} %{SOURCE10} %{SOURCE12} %{SOURCE18} %{SOURCE220} .

%if 0%{?rhel} > 0 && 0%{?rhel} < 8
sed -i -e 's#KillMode=.*#KillMode=process#g' freenginx.service freenginx@.service
sed -i -e 's#PROFILE=SYSTEM#HIGH:!aNULL:!MD5#' freenginx.conf
%endif

%if 0%{?openssl_pkgversion}
sed \
  -e 's|\(ngx_feature_path=\)$|\1%{_includedir}/openssl%{openssl_pkgversion}|' \
  -e 's|\(ngx_feature_libs="\)|\1-L%{_libdir}/openssl%{openssl_pkgversion} |' \
  -i auto/lib/openssl/conf
%endif

# Prepare template config for instances
sed -e '/^error_log /s|error\.log|@INSTANCE@_error.log|' \
    -e '/^pid /s|freenginx\.pid|freenginx-@INSTANCE@.pid|' \
    -e '/^ *access_log/s|access\.log|@INSTANCE@_access.log|' \
    freenginx.conf >> instance.conf
touch -r %{SOURCE12} instance.conf

# Prepare sources for installation
cp -a ../%{name}-%{version} ../%{name}-%{version}-%{release}-src
mv ../%{name}-%{version}-%{release}-src .


%build
# freenginx does not utilize a standard configure script.  It has its own
# and the standard configure options cause the freenginx configure script
# to error out.  This is is also the reason for the DESTDIR environment
# variable.
export DESTDIR=%{buildroot}
# So the perl module finds its symbols:
freenginx_ldopts="$RPM_LD_FLAGS -Wl,-E -O2"
if ! ./configure \
    --prefix=%{_datadir}/freenginx \
    --sbin-path=%{_sbindir}/freenginx \
    --modules-path=%{freenginx_moduledir} \
    --conf-path=%{_sysconfdir}/freenginx/freenginx.conf \
    --error-log-path=%{_localstatedir}/log/freenginx/error.log \
    --http-log-path=%{_localstatedir}/log/freenginx/access.log \
    --http-client-body-temp-path=%{_localstatedir}/lib/freenginx/tmp/client_body \
    --http-proxy-temp-path=%{_localstatedir}/lib/freenginx/tmp/proxy \
    --http-fastcgi-temp-path=%{_localstatedir}/lib/freenginx/tmp/fastcgi \
    --http-uwsgi-temp-path=%{_localstatedir}/lib/freenginx/tmp/uwsgi \
    --http-scgi-temp-path=%{_localstatedir}/lib/freenginx/tmp/scgi \
    --pid-path=/run/freenginx.pid \
    --lock-path=/run/lock/subsys/freenginx \
    --user=%{freenginx_user} \
    --group=%{freenginx_user} \
    --with-compat \
    --with-debug \
%if 0%{?with_aio}
    --with-file-aio \
%endif
%if 0%{?with_gperftools}
    --with-google_perftools_module \
%endif
    --with-http_addition_module \
    --with-http_auth_request_module \
    --with-http_dav_module \
    --with-http_degradation_module \
    --with-http_flv_module \
%if %{with geoip}
    --with-http_geoip_module=dynamic \
    --with-stream_geoip_module=dynamic \
%endif
    --with-http_gunzip_module \
    --with-http_gzip_static_module \
    --with-http_image_filter_module \
    --with-http_mp4_module \
    --with-http_perl_module \
    --with-http_random_index_module \
    --with-http_realip_module \
    --with-http_secure_link_module \
    --with-http_slice_module \
    --with-http_ssl_module \
    --with-http_stub_status_module \
    --with-http_sub_module \
    --with-http_v2_module \
    --with-http_v3_module \
    --with-http_xslt_module \
    --with-mail \
    --with-mail_ssl_module \
%if 0%{?with_ktls}
    --with-openssl-opt=enable-ktls \
%endif
    --with-pcre \
    --with-pcre-jit \
    --with-stream \
    --with-stream_realip_module \
    --with-stream_ssl_module \
    --with-stream_ssl_preread_module \
    --with-threads \
    --with-cc-opt="%{optflags} $(pcre2-config --cflags)" \
    --with-ld-opt="$freenginx_ldopts"; then
  : configure failed
  cat objs/autoconf.err
  exit 1
fi

%make_build


%install
%make_install INSTALLDIRS=vendor

find %{buildroot} -type f -name .packlist -exec rm -f '{}' \;
find %{buildroot} -type f -name perllocal.pod -exec rm -f '{}' \;
find %{buildroot} -type f -empty -exec rm -f '{}' \;
find %{buildroot} -type f -iname '*.so' -exec chmod 0755 '{}' \;

install -p -D -m 0644 ./freenginx.service \
    %{buildroot}%{_unitdir}/freenginx.service
install -p -D -m 0644 ./freenginx@.service \
    %{buildroot}%{_unitdir}/freenginx@.service
install -p -D -m 0644 %{SOURCE11} \
    %{buildroot}%{_sysconfdir}/logrotate.d/freenginx

install -p -d -m 0755 %{buildroot}%{_sysconfdir}/systemd/system/freenginx.service.d
install -p -d -m 0755 %{buildroot}%{_unitdir}/freenginx.service.d

install -p -d -m 0755 %{buildroot}%{_sysconfdir}/freenginx/conf.d
install -p -d -m 0755 %{buildroot}%{_sysconfdir}/freenginx/default.d

install -p -d -m 0700 %{buildroot}%{_localstatedir}/lib/freenginx
install -p -d -m 0700 %{buildroot}%{_localstatedir}/lib/freenginx/tmp
install -p -d -m 0700 %{buildroot}%{_localstatedir}/log/freenginx

install -p -d -m 0755 %{buildroot}%{_datadir}/freenginx/html
install -p -d -m 0755 %{buildroot}%{freenginx_moduleconfdir}
install -p -d -m 0755 %{buildroot}%{freenginx_moduledir}

install -p -m 0644 ./freenginx.conf \
    %{buildroot}%{_sysconfdir}/freenginx

rm -f %{buildroot}%{_datadir}/freenginx/html/index.html
%if 0%{?el7}
ln -s ../../doc/HTML/index.html \
      %{buildroot}%{_datadir}/freenginx/html/index.html
ln -s ../../doc/HTML/img \
      %{buildroot}%{_datadir}/freenginx/html/img
ln -s ../../doc/HTML/en-US \
      %{buildroot}%{_datadir}/freenginx/html/en-US
%else
ln -s ../../testpage/index.html \
      %{buildroot}%{_datadir}/freenginx/html/index.html
%endif
install -p -m 0644 %{SOURCE102} \
    %{buildroot}%{_datadir}/freenginx/html
ln -s freenginx-logo.png %{buildroot}%{_datadir}/freenginx/html/poweredby.png
mkdir -p %{buildroot}%{_datadir}/freenginx/html/icons

# Symlink for the powered-by-$DISTRO image:
ln -s ../../../pixmaps/poweredby.png \
      %{buildroot}%{_datadir}/freenginx/html/icons/poweredby.png

%if 0%{?rhel} >= 9
ln -s ../../pixmaps/system-noindex-logo.png \
      %{buildroot}%{_datadir}/freenginx/html/system_noindex_logo.png
%endif

%if 0%{?with_mailcap_mimetypes}
rm -f %{buildroot}%{_sysconfdir}/freenginx/mime.types
%endif

install -p -D -m 0644 %{_builddir}/freenginx-%{version}/objs/nginx.8 \
    %{buildroot}%{_mandir}/man8/freenginx.8

install -p -D -m 0755 %{SOURCE13} %{buildroot}%{_bindir}/freenginx-upgrade
install -p -D -m 0644 %{SOURCE14} %{buildroot}%{_mandir}/man8/freenginx-upgrade.8

for i in ftdetect ftplugin indent syntax; do
    install -p -D -m644 contrib/vim/${i}/nginx.vim \
        %{buildroot}%{_datadir}/vim/vimfiles/${i}/freenginx.vim
done

%if %{with geoip}
echo 'load_module "%{freenginx_moduledir}/ngx_http_geoip_module.so";' \
    > %{buildroot}%{freenginx_moduleconfdir}/mod-http-geoip.conf
%endif

# Install files for supporting freenginx module builds
## Install source files
mkdir -p %{buildroot}%{_usrsrc}
mv %{name}-%{version}-%{release}-src %{buildroot}%{freenginx_srcdir}
## Install rpm macros
mkdir -p %{buildroot}%{_rpmmacrodir}
sed -e "s|@@freenginx_ABIVERSION@@|%{freenginx_abiversion}|g" \
    -e "s|@@freenginx_MODDIR@@|%{freenginx_moduledir}|g" \
    -e "s|@@freenginx_MODCONFDIR@@|%{freenginx_moduleconfdir}|g" \
    -e "s|@@freenginx_SRCDIR@@|%{freenginx_srcdir}|g" \
    %{SOURCE15} > %{buildroot}%{_rpmmacrodir}/macros.freenginxmods
## Install dependency generator
install -Dpm0644 %{SOURCE16} %{buildroot}%{_fileattrsdir}/freenginxmods.attr

# install http-ssl-pass-dialog
mkdir -p $RPM_BUILD_ROOT%{_libexecdir}
install -m755 $RPM_SOURCE_DIR/freenginx-ssl-pass-dialog \
        $RPM_BUILD_ROOT%{_libexecdir}/freenginx-ssl-pass-dialog

# install sysusers file
install -p -D -m 0644 %{SOURCE19} %{buildroot}%{_sysusersdir}/freenginx.conf

# rename to freenginx
mv %{buildroot}%{_mandir}/man3/nginx.3pm %{buildroot}%{_mandir}/man3/freenginx.3pm
mv %{buildroot}%{_sysconfdir}/freenginx/nginx.conf.default %{buildroot}%{_sysconfdir}/freenginx/freenginx.conf.default

%pre filesystem
%sysusers_create_compat %{SOURCE19}

%post
%systemd_post freenginx.service

%if %{with geoip}
%post mod-http-geoip
if [ $1 -eq 1 ]; then
    /usr/bin/systemctl reload freenginx.service >/dev/null 2>&1 || :
fi
%endif

%preun
%systemd_preun freenginx.service

%postun
%systemd_postun freenginx.service
if [ $1 -ge 1 ]; then
    /usr/bin/freenginx-upgrade >/dev/null 2>&1 || :
fi

%files
%{_datadir}/freenginx/html/*
%{_bindir}/freenginx-upgrade
%{_datadir}/vim/vimfiles/ftdetect/freenginx.vim
%{_datadir}/vim/vimfiles/ftplugin/freenginx.vim
%{_datadir}/vim/vimfiles/syntax/freenginx.vim
%{_datadir}/vim/vimfiles/indent/freenginx.vim
%{_mandir}/man3/freenginx.3pm*
%{_mandir}/man8/freenginx.8*
%{_mandir}/man8/freenginx-upgrade.8*
%{_unitdir}/freenginx.service
%{_unitdir}/freenginx@.service
%{_libexecdir}/freenginx-ssl-pass-dialog

%files core
%license LICENSE
%doc CHANGES README README.dynamic instance.conf
%{_sbindir}/freenginx
%config(noreplace) %{_sysconfdir}/freenginx/fastcgi.conf
%config(noreplace) %{_sysconfdir}/freenginx/fastcgi.conf.default
%config(noreplace) %{_sysconfdir}/freenginx/fastcgi_params
%config(noreplace) %{_sysconfdir}/freenginx/fastcgi_params.default
%config(noreplace) %{_sysconfdir}/freenginx/koi-utf
%config(noreplace) %{_sysconfdir}/freenginx/koi-win
%if ! 0%{?with_mailcap_mimetypes}
%config(noreplace) %{_sysconfdir}/freenginx/mime.types
%endif
%config(noreplace) %{_sysconfdir}/freenginx/mime.types.default
%config(noreplace) %{_sysconfdir}/freenginx/freenginx.conf
%config(noreplace) %{_sysconfdir}/freenginx/freenginx.conf.default
%config(noreplace) %{_sysconfdir}/freenginx/scgi_params
%config(noreplace) %{_sysconfdir}/freenginx/scgi_params.default
%config(noreplace) %{_sysconfdir}/freenginx/uwsgi_params
%config(noreplace) %{_sysconfdir}/freenginx/uwsgi_params.default
%config(noreplace) %{_sysconfdir}/freenginx/win-utf
%config(noreplace) %{_sysconfdir}/logrotate.d/freenginx
%attr(770,%{freenginx_user},root) %dir %{_localstatedir}/lib/freenginx
%attr(770,%{freenginx_user},root) %dir %{_localstatedir}/lib/freenginx/tmp
%attr(711,root,root) %dir %{_localstatedir}/log/freenginx
%ghost %attr(640,%{freenginx_user},root) %{_localstatedir}/log/freenginx/access.log
%ghost %attr(640,%{freenginx_user},root) %{_localstatedir}/log/freenginx/error.log
%dir %{freenginx_moduledir}
%dir %{freenginx_moduleconfdir}
%dir %{perl_vendorarch}/auto/nginx
%{perl_vendorarch}/nginx.pm
%{perl_vendorarch}/auto/nginx/nginx.so

%files filesystem
%dir %{_datadir}/freenginx
%dir %{_datadir}/freenginx/html
%dir %{_sysconfdir}/freenginx
%dir %{_sysconfdir}/freenginx/conf.d
%dir %{_sysconfdir}/freenginx/default.d
%dir %{_sysconfdir}/systemd/system/freenginx.service.d
%dir %{_unitdir}/freenginx.service.d
%{_sysusersdir}/freenginx.conf

%if %{with geoip}
%files mod-http-geoip
%{freenginx_moduleconfdir}/00-mod-http-geoip.conf
%{freenginx_moduledir}/ngx_http_geoip_module.so
%endif

%files mod-devel
%{_rpmmacrodir}/macros.freenginxmods
%{_fileattrsdir}/freenginxmods.attr
%{freenginx_srcdir}/


%changelog
* Thu Apr 24 2025 Teddysun <i@teddysun.com> - 2:1.28.0-1
- update to 1.28.0
- http_image_filter_module is not dynamic module anymore
- http_perl_module is not dynamic module anymore
- http_xslt_filter_module is not dynamic module anymore
- mail_module is not dynamic module anymore
- stream_module is not dynamic module anymore

* Thu Apr 17 2025 Teddysun <i@teddysun.com> - 2:1.27.6-1
- update to 1.27.6

* Sun Apr 06 2025 Teddysun <i@teddysun.com> - 2:1.27.4-3
- update to 1.27.4
- use Fedora spec to rebuild
